@@ -1,5 +1,7 @@
 import asyncio
-from typing import Any, AsyncGenerator, Generator, List, Type, Union
+from typing import Generator, List, Type, Union, Dict
+import inspect
+import traceback
 
 from iterwrapper import IterWrapper as iw
 
@@ -13,16 +15,14 @@ from .entities.namespace import Namespace
 from .entities.signatures import Force, RemoveMe
 from .exceptions import (DisabledNamespace, ExistedNamespace,
                          RegisteredEventListener, RequirementCrashed,
-                         UnexistedNamespace, InvaildEventName)
+                         UnexistedNamespace, InvaildEventName, PropagationCancelled)
 from .interfaces.decorater import DecoraterInterface
 from .interfaces.dispatcher import DispatcherInterface
 from .protocols.executor import ExecutorProtocol
-from .utilles import argument_signature, iw_group, printer, run_always_await
-
+from .utilles import argument_signature, run_always_await, whatever_gen_once, group_dict
 
 class Broadcast:
   loop: asyncio.AbstractEventLoop
-  event_queue: asyncio.Queue
 
   default_namespace: Namespace
   namespaces: List[Namespace] = []
@@ -30,38 +30,34 @@ class Broadcast:
 
   stoped: bool = False
 
-  def __init__(self, *, loop: asyncio.AbstractEventLoop = None, queue: asyncio.Queue = None):
+  def __init__(self, *, loop: asyncio.AbstractEventLoop = None):
     self.loop = loop or asyncio.get_event_loop()
-    self.event_queue = queue or asyncio.Queue(15, loop=self.loop)
     self.default_namespace = Namespace(name="default", default=True)
-
-  async def event_generator(self) -> AsyncGenerator[Any, BaseEvent]:
-      while True:
-        try:
-          yield await asyncio.wait_for(self.event_queue.get(), timeout=5)
-        except asyncio.TimeoutError:
-          continue
-        except asyncio.CancelledError:
-          return
   
-  def listener_generator(self, event_class) -> Listener:
-    yield from (iw_group(iw(self.listeners)
+  def default_listener_generator(self, event_class) -> Listener:
+    yield from (iw(self.listeners)
       .filter(lambda x: not x.namespace.hide) # filter for hide
       .filter(lambda x: not x.namespace.disabled) # filter for disabled
-      .filter(lambda x: event_class in x.listening_events),
-        key=lambda x: x.namespace.name # group
-      ).map(lambda x: iw(x).apply(list.sort, key=lambda x: x.priority).collect(list)) # sort in group
-      .flat() # unpack
+      .filter(lambda x: event_class in x.listening_events)
       .collect(list) # collect to a whole list
     )
 
-  async def event_runner(self):
-    async for event in self.event_generator():
-      for listener in self.listener_generator(event.__class__):
-        self.loop.create_task(self.Executor(ExecutorProtocol(
-          target=listener,
+  async def layered_scheduler(self,
+    listener_generator: Generator[Listener, None, None],
+    event: BaseEvent
+  ):
+    listeners = list(listener_generator) # 此处的 listeners 是已经被筛过的了
+    grouped: Dict[int, List[Listener]] = group_dict(listeners, lambda x: x.priority)
+    for current_priority in sorted(grouped.keys()):
+      current_group = grouped[current_priority]
+      tasks, _ = await asyncio.wait([
+        self.Executor(ExecutorProtocol(
+          target=i,
           event=event
-        )))
+        )) for i in current_group
+      ])
+      if any([True if isinstance(i.exception(), PropagationCancelled) else False for i in list(tasks)]):
+        break
       
   async def Executor(self, protocol: ExecutorProtocol):
     if isinstance(protocol.target, Listener):
@@ -83,10 +79,11 @@ class Broadcast:
     parameter_compile_result = {}
 
     async with DispatcherInterface(self, protocol.event, dispatchers) as dii:
-      dei = DecoraterInterface(dii)  # pylint: disable=unused-variable
+      if (not dii.dispatchers) or type(dii.dispatchers[0]) is not DecoraterInterface:
+        DecoraterInterface(dii)  # pylint: disable=unused-variable
       # Decorater 的 Dispatcher 已经注入, 没他事了
 
-      dii.dispatchers.append(SimpleMapping([
+      dii.inject_global_dispatcher(SimpleMapping([
         MappingRule.annotationEquals(Broadcast, self),
         MappingRule.annotationEquals(ExecutorProtocol, protocol),
         MappingRule.annotationEquals(DispatcherInterface, dii),
@@ -95,26 +92,39 @@ class Broadcast:
           MappingRule.annotationEquals(Namespace, protocol.target.namespace)
         ] if isinstance(protocol.target, Listener) else []),
       ]))
+
       try:
         for name, annotation, default in argument_signature(target_callable):
           parameter_compile_result[name] =\
             await dii.execute_with(name, annotation, default)
       except RequirementCrashed:
-            raise
+        raise
       except Exception as e:
-        import traceback
         traceback.print_exc()
-        if not protocol.hasReferrer:
-          await self.postEvent(ExceptionThrowed(
+        if not protocol.hasReferrer: # 如果没有referrer, 会广播事件, 如果有则向上抛出
+          self.postEvent(ExceptionThrowed(
             exception=e,
             event=protocol.event
           ))
-        else:
-          raise
+        raise
 
-      result = await run_always_await(
-        target_callable(**parameter_compile_result)
-      )
+      try:
+        result = await run_always_await(
+          target_callable(**parameter_compile_result)
+        )
+      except PropagationCancelled:
+        raise # 防止打印出错误
+      except Exception as e:
+        traceback.print_exc()
+        if not protocol.hasReferrer: # 如果没有referrer, 会广播事件, 如果有则向上抛出
+          self.postEvent(ExceptionThrowed(
+            exception=e,
+            event=protocol.event
+          ))
+        raise
+      if inspect.isgenerator(result) or inspect.isasyncgen(result):
+        dii.alive_generater_dispatcher[-1].append(result)
+        result = await whatever_gen_once(result)
       if isinstance(result, Force):
         return result.content
 
@@ -124,8 +134,11 @@ class Broadcast:
             self.listeners.pop(self.listeners.index(protocol.target))
       return result
 
-  async def postEvent(self, event: BaseEvent):
-    await self.event_queue.put(event)
+  def postEvent(self, event: BaseEvent):
+    self.loop.create_task(self.layered_scheduler(
+      listener_generator=self.default_listener_generator(event.__class__),
+      event=event
+    ))
 
   @staticmethod
   def event_class_generator(target=BaseEvent):
@@ -151,10 +164,13 @@ class Broadcast:
   
   def removeNamespace(self, name):
     if self.containNamespace(name):
+      self.listeners = [i for i in self.listeners if i.namespace.name != name]
       for index, i in enumerate(self.namespaces):
         if i.name == name:
           self.namespaces.pop(index)
           return
+    else:
+      raise UnexistedNamespace(name)
 
   def containNamespace(self, name):
     for i in self.namespaces:
@@ -167,6 +183,8 @@ class Broadcast:
       for i in self.namespaces:
         if i.name == name:
           return i
+    else:
+      raise UnexistedNamespace(name)
 
   def hideNamespace(self, name):
     ns = self.getNamespace(name)
@@ -195,9 +213,12 @@ class Broadcast:
       if i.callable == target:
         return i
 
+  def removeListener(self, target):
+    self.listeners.remove(target)
+
   def receiver(self,
       event: Union[str, Type[BaseEvent]],
-      priority: int = 0,
+      priority: int = 16,
       dispatchers: List[Type[BaseDispatcher]] = [],
       namespace: Namespace = None
   ):
@@ -206,7 +227,7 @@ class Broadcast:
       event = self.findEvent(event)
       if not event:
         raise InvaildEventName(_name, "is not vaild!")
-    print(event)
+    priority = (type(priority) == int) and priority or int(priority) # 类型转换
     def receiver_wrapper(callable_target):
       may_listener = self.getListener(callable_target)
       if not may_listener:
