@@ -1,5 +1,5 @@
 import inspect
-from typing import AsyncGenerator, Generator, List, Union
+from typing import AsyncGenerator, Generator, List, Union, Any, Callable
 
 from iterwrapper import IterWrapper
 
@@ -10,19 +10,59 @@ from ..exceptions import (InvaildDispatcher, OutOfMaxGenerater,
                           RequirementCrashed)
 from ..utilles import async_enumerate
 
+class ContextStackItem:
+  def __init__(self, name, annotation, default, local_dispatchers, index=0) -> None:
+    self.name, self.annotation, self.default, self.local_dispatchers, self.index = \
+      name, annotation, default, local_dispatchers, index
+  
+  def __repr__(self) -> str:
+    return "<ContextStackItem name={0} annotation={1} default={2} locald={3} index={4}>"\
+      .format(
+        self.name, self.annotation, self.default, self.local_dispatchers, self.index)
+
+  name: str
+  annotation: Any
+  default: Any
+  local_dispatchers: List[Union[BaseDispatcher, Callable]]
+  index: int
+  
 
 class DispatcherInterface:
   broadcast: "Broadcast"
   event: BaseEvent
   dispatchers: List[BaseDispatcher]
 
-  alive_generater_dispatcher: List[Union[Generator, AsyncGenerator]] = []
+  # name, annotation, default, local_dispatchers, index
+  context_stack: List[ContextStackItem] = [ContextStackItem(None, None, None, [], 0)]
+  alive_generater_dispatcher: List[List[Union[Generator, AsyncGenerator]]]
 
   _index = 0
 
-  name: str = None
-  annotation = None
-  default = None # dispatcher 允许任何值, 包括 Decorater, 只不过 Decorater Dispatcher 的优先级是隐式最高, 所以基本上不可能截获到.
+  @property
+  def name(self):
+    return self.context_stack[-1].name
+
+  @property
+  def annotation(self):
+    return self.context_stack[-1].annotation
+
+  @property
+  def default(self):
+    return self.context_stack[-1].default
+
+  @property
+  def _index(self):
+    return self.context_stack[-1].index
+
+  @_index.setter
+  def _(self, new_value):
+    self.context_stack[-1].index = new_value
+
+  @property
+  def local_dispatchers(self):
+    return self.context_stack[-1].local_dispatchers
+  
+  # dispatcher 允许任何值, 包括 Decorater, 只不过 Decorater Dispatcher 的优先级是隐式最高, 所以基本上不可能截获到.
 
   def __init__(self,
     broadcast_instance: "Broadcast",
@@ -32,8 +72,9 @@ class DispatcherInterface:
     self.broadcast = broadcast_instance
     self.event = event_instance
     self.dispatchers = dispatchers
+    self.alive_generater_dispatcher = [[]]
 
-  async def __aenter__(self) -> "DispatchInterface":
+  async def __aenter__(self) -> "DispatcherInterface":
     return self
 
   async def __aexit__(self, exc_type, exc, tb):
@@ -41,18 +82,24 @@ class DispatcherInterface:
     if tb is not None:
       raise exc
 
+  def inject_local_dispatcher(self, *dispatchers: List[Union[BaseDispatcher, Callable]]):
+    self.local_dispatchers.extend(dispatchers)
+
+  def inject_global_dispatcher(self, *dispatchers: List[Union[BaseDispatcher, Callable]]):
+    self.dispatchers.extend(dispatchers)
+
   async def execute_with(self, name: str, annotation, default):
     # here, dispatcher.mixins has been handled.
-    initial_value = (None,) * 3 + (0,)
-
-    if any([self.name is not None, self.annotation is not None, self.default is not None, self._index != 0]):
-      initial_value = (self.name, self.annotation, self.default, self._index)
-    self.name, self.annotation, self.default = name, annotation, default
+    
+    # create a new context
+    # generater has special handle method
+    self.context_stack.append(ContextStackItem(name, annotation, default, []))
+    self.alive_generater_dispatcher.append([])
     try:
-      for self._index, dispatcher in enumerate(
+      for self.context_stack[-1].index, dispatcher in enumerate(
           self.dispatchers[self._index:], start=self._index):
 
-        # just allow Dispatcher or any Callable
+        # just allow Dispatcher(class or instance) or any Callable
         if not isinstance(dispatcher, BaseDispatcher) and not callable(dispatcher):
             raise InvaildDispatcher("dispatcher must base on 'BaseDispatcher' or is a callable: ", dispatcher)
           
@@ -83,7 +130,7 @@ class DispatcherInterface:
               result = now_dispatcher_generater.__next__()
             except StopIteration as e:
               result = e.value
-          self.alive_generater_dispatcher.append(now_dispatcher_generater)
+          self.alive_generater_dispatcher[-1].append(now_dispatcher_generater)
         else:
           if inspect.iscoroutinefunction(local_dispatcher):
             result = await local_dispatcher(self)
@@ -100,10 +147,10 @@ class DispatcherInterface:
       else:
         raise RequirementCrashed("the dispatching requirement crashed: ", self.name, self.annotation, self.default)
     finally:
-        self.name, self.annotation, self.default, self._index = initial_value
+      self.context_stack.pop()
 
   async def alive_dispatcher_killer(self):
-    for unbound_gen in self.alive_generater_dispatcher:
+    for unbound_gen in self.alive_generater_dispatcher[-1]:
       if inspect.isgenerator(unbound_gen):
         for index, _ in enumerate(unbound_gen, start=1):
           if index == 16:
@@ -127,4 +174,4 @@ class DispatcherInterface:
       .filter(lambda x: issubclass(x, BaseDispatcher))
       .map(lambda x: DispatcherInterface.dispatcher_mixin_handler(x))
       .flat().collect(list)
-    )]
+    )]
\ No newline at end of file
